@@ -11,27 +11,38 @@ from typing import (
     Dict,
     List,
     Optional,
+    overload,
+    Tuple,
+    TYPE_CHECKING,
     Union,
 )
 from urllib.parse import urljoin
 
+from requests import Response
+from typing_extensions import Literal
+
 import bioblend
 from bioblend import TimeoutException
 from bioblend.galaxy.client import Client
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
 log = logging.getLogger(__name__)
 
+HdaLdda = Literal["hda", "ldda"]
 TERMINAL_STATES = {"ok", "empty", "error", "discarded", "failed_metadata"}
 # Non-terminal states are: 'new', 'upload', 'queued', 'running', 'paused', 'setting_metadata'
 
 
 class DatasetClient(Client):
+    gi: "GalaxyInstance"
     module = "datasets"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def show_dataset(self, dataset_id, deleted=False, hda_ldda="hda"):
+    def show_dataset(self, dataset_id: str, deleted: bool = False, hda_ldda: HdaLdda = "hda") -> Dict[str, Any]:
         """
         Get details about a given dataset. This can be a history or a library dataset.
 
@@ -55,14 +66,14 @@ class DatasetClient(Client):
 
     def _initiate_download(
         self, dataset_id: str, stream_content: bool, require_ok_state: bool = True, maxwait: float = 12000
-    ):
+    ) -> Tuple[Dict[str, Any], str, Response]:
         dataset = self.wait_for_dataset(dataset_id, maxwait=maxwait, check=False)
         if not dataset["state"] == "ok":
             message = f"Dataset state is not 'ok'. Dataset id: {dataset_id}, current state: {dataset['state']}"
             if require_ok_state:
                 raise DatasetStateException(message)
             else:
-                warnings.warn(message, DatasetStateWarning)
+                warnings.warn(message, DatasetStateWarning, stacklevel=2)
 
         file_ext = dataset.get("file_ext")
         # Resort to 'data' when Galaxy returns an empty or temporary extension
@@ -81,9 +92,36 @@ class DatasetClient(Client):
         r.raise_for_status()
         return dataset, file_ext, r
 
+    @overload
     def download_dataset(
-        self, dataset_id, file_path=None, use_default_filename=True, require_ok_state=True, maxwait=12000
-    ):
+        self,
+        dataset_id: str,
+        file_path: None = None,
+        use_default_filename: bool = True,
+        require_ok_state: bool = True,
+        maxwait: float = 12000,
+    ) -> bytes:
+        ...
+
+    @overload
+    def download_dataset(
+        self,
+        dataset_id: str,
+        file_path: str,
+        use_default_filename: bool = True,
+        require_ok_state: bool = True,
+        maxwait: float = 12000,
+    ) -> str:
+        ...
+
+    def download_dataset(
+        self,
+        dataset_id: str,
+        file_path: Optional[str] = None,
+        use_default_filename: bool = True,
+        require_ok_state: bool = True,
+        maxwait: float = 12000,
+    ) -> Union[bytes, str]:
         """
         Download a dataset to file or in memory. If the dataset state is not
         'ok', a ``DatasetStateException`` will be thrown, unless ``require_ok_state=False``.
@@ -167,12 +205,12 @@ class DatasetClient(Client):
         tool_id: Optional[str] = None,
         tag: Optional[str] = None,
         history_id: Optional[str] = None,
-        create_time_min: str = None,
-        create_time_max: str = None,
-        update_time_min: str = None,
-        update_time_max: str = None,
+        create_time_min: Optional[str] = None,
+        create_time_max: Optional[str] = None,
+        update_time_min: Optional[str] = None,
+        update_time_max: Optional[str] = None,
         order: str = "create_time-dsc",
-    ) -> List[dict]:
+    ) -> List[Dict[str, Any]]:
         """
         Get the latest datasets, or select another subset by specifying optional
         arguments for filtering (e.g. a history ID).
@@ -253,7 +291,7 @@ class DatasetClient(Client):
             params["history_id"] = history_id
 
         q: List[str] = []
-        qv: List[Any] = []
+        qv = []
 
         if name:
             q.append("name")
@@ -299,16 +337,16 @@ class DatasetClient(Client):
 
         return self._get(params=params)
 
-    def _param_to_filter(self, param):
-        if type(param) is str:
+    def _param_to_filter(self, param: Union[str, List[str]]) -> Tuple[str, str]:
+        if isinstance(param, str):
             return "eq", param
-        if type(param) is list:
+        if isinstance(param, list):
             if len(param) == 1:
                 return "eq", param.pop()
             return "in", ",".join(param)
         raise Exception("Filter param is not of type ``str`` or ``list``")
 
-    def publish_dataset(self, dataset_id: str, published: bool = False):
+    def publish_dataset(self, dataset_id: str, published: bool = False) -> Dict[str, Any]:
         """
         Make a dataset publicly available or private. For more fine-grained control (assigning different
         permissions to specific roles), use the ``update_permissions()`` method.
@@ -320,14 +358,14 @@ class DatasetClient(Client):
         :param published: Whether to make the dataset published (``True``) or private (``False``).
 
         :rtype: dict
-        :return: Current roles for all available permission types.
+        :return: Details of the updated dataset
 
         .. note::
           This method works only on Galaxy 19.05 or later.
         """
         payload: Dict[str, Any] = {"action": "remove_restrictions" if published else "make_private"}
         url = self._make_url(dataset_id) + "/permissions"
-        self.gi.datasets._put(url=url, payload=payload)
+        return self.gi.datasets._put(url=url, payload=payload)
 
     def update_permissions(
         self,
@@ -335,7 +373,7 @@ class DatasetClient(Client):
         access_ids: Optional[list] = None,
         manage_ids: Optional[list] = None,
         modify_ids: Optional[list] = None,
-    ):
+    ) -> dict:
         """
         Set access, manage or modify permissions for a dataset to a list of roles.
 
@@ -365,9 +403,11 @@ class DatasetClient(Client):
         if modify_ids:
             payload["modify"] = modify_ids
         url = self._make_url(dataset_id) + "/permissions"
-        self.gi.datasets._put(url=url, payload=payload)
+        return self.gi.datasets._put(url=url, payload=payload)
 
-    def wait_for_dataset(self, dataset_id, maxwait=12000, interval=3, check=True):
+    def wait_for_dataset(
+        self, dataset_id: str, maxwait: float = 12000, interval: float = 3, check: bool = True
+    ) -> Dict[str, Any]:
         """
         Wait until a dataset is in a terminal state.
 
@@ -413,7 +453,7 @@ class DatasetStateException(Exception):
     pass
 
 
-class DatasetStateWarning(Warning):
+class DatasetStateWarning(UserWarning):
     pass
 
 
