@@ -3,7 +3,15 @@ Contains possible interactions with the Galaxy Data Libraries
 """
 import logging
 import time
-import warnings
+from typing import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    TYPE_CHECKING,
+)
+
+from typing_extensions import Literal
 
 from bioblend.galaxy.client import Client
 from bioblend.galaxy.datasets import (
@@ -12,16 +20,23 @@ from bioblend.galaxy.datasets import (
 )
 from bioblend.util import attach_file
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
+LinkDataOnly = Literal["copy_files", "link_to_files"]
+
 log = logging.getLogger(__name__)
 
 
 class LibraryClient(Client):
     module = "libraries"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def create_library(self, name, description=None, synopsis=None):
+    def create_library(
+        self, name: str, description: Optional[str] = None, synopsis: Optional[str] = None
+    ) -> Dict[str, Any]:
         """
         Create a data library with the properties defined in the arguments.
 
@@ -49,7 +64,7 @@ class LibraryClient(Client):
             payload["synopsis"] = synopsis
         return self._post(payload)
 
-    def delete_library(self, library_id):
+    def delete_library(self, library_id: str) -> Dict[str, Any]:
         """
         Delete a data library.
 
@@ -66,14 +81,14 @@ class LibraryClient(Client):
         """
         return self._delete(id=library_id)
 
-    def _show_item(self, library_id, item_id):
+    def _show_item(self, library_id: str, item_id: str) -> Dict[str, Any]:
         """
         Get details about a given library item.
         """
         url = "/".join((self._make_url(library_id, contents=True), item_id))
         return self._get(url=url)
 
-    def delete_library_dataset(self, library_id, dataset_id, purged=False):
+    def delete_library_dataset(self, library_id: str, dataset_id: str, purged: bool = False) -> Dict[str, Any]:
         """
         Delete a library dataset in a data library.
 
@@ -98,7 +113,7 @@ class LibraryClient(Client):
         url = "/".join((self._make_url(library_id, contents=True), dataset_id))
         return self._delete(payload={"purged": purged}, url=url)
 
-    def update_library_dataset(self, dataset_id, **kwds):
+    def update_library_dataset(self, dataset_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Update library dataset metadata. Some of the attributes that can be
         modified are documented below.
@@ -125,9 +140,9 @@ class LibraryClient(Client):
         :return: details of the updated dataset
         """
         url = "/".join((self._make_url(), "datasets", dataset_id))
-        return self._patch(payload=kwds, url=url)
+        return self._patch(payload=kwargs, url=url)
 
-    def show_dataset(self, library_id, dataset_id):
+    def show_dataset(self, library_id: str, dataset_id: str) -> Dict[str, Any]:
         """
         Get details about a given library dataset. The required ``library_id``
         can be obtained from the datasets's library content details.
@@ -144,7 +159,9 @@ class LibraryClient(Client):
         """
         return self._show_item(library_id, dataset_id)
 
-    def wait_for_dataset(self, library_id, dataset_id, maxwait=12000, interval=3):
+    def wait_for_dataset(
+        self, library_id: str, dataset_id: str, maxwait: float = 12000, interval: float = 3
+    ) -> Dict[str, Any]:
         """
         Wait until the library dataset state is terminal ('ok', 'empty',
         'error', 'discarded' or 'failed_metadata').
@@ -191,7 +208,7 @@ class LibraryClient(Client):
                     f"Waited too long for dataset {dataset_id} in library {library_id} to complete"
                 )
 
-    def show_folder(self, library_id, folder_id):
+    def show_folder(self, library_id: str, folder_id: str) -> Dict[str, Any]:
         """
         Get details about a given folder. The required ``folder_id`` can be
         obtained from the folder's library content details.
@@ -207,17 +224,19 @@ class LibraryClient(Client):
         """
         return self._show_item(library_id, folder_id)
 
-    def _get_root_folder_id(self, library_id):
+    def _get_root_folder_id(self, library_id: str) -> str:
         """
         Find the root folder (i.e. '/') of a library.
 
         :type library_id: str
         :param library_id: library id to find root of
         """
-        l = self.show_library(library_id=library_id)
-        return l["root_folder_id"]
+        library_dict = self.show_library(library_id=library_id)
+        return library_dict["root_folder_id"]
 
-    def create_folder(self, library_id, folder_name, description=None, base_folder_id=None):
+    def create_folder(
+        self, library_id: str, folder_name: str, description: Optional[str] = None, base_folder_id: Optional[str] = None
+    ) -> List[Dict[str, Any]]:
         """
         Create a folder in a library.
 
@@ -241,15 +260,18 @@ class LibraryClient(Client):
         if base_folder_id is None:
             base_folder_id = self._get_root_folder_id(library_id)
         # Compose the payload
-        payload = {}
-        payload["name"] = folder_name
-        payload["folder_id"] = base_folder_id
-        payload["create_type"] = "folder"
+        payload = {
+            "name": folder_name,
+            "folder_id": base_folder_id,
+            "create_type": "folder",
+        }
         if description is not None:
             payload["description"] = description
         return self._post(payload, id=library_id, contents=True)
 
-    def get_folders(self, library_id, folder_id=None, name=None):
+    def get_folders(
+        self, library_id: str, folder_id: Optional[str] = None, name: Optional[str] = None
+    ) -> List[Dict[str, Any]]:
         """
         Get all the folders in a library, or select a subset by specifying a
         folder name for filtering.
@@ -257,13 +279,6 @@ class LibraryClient(Client):
         :type library_id: str
         :param library_id: library id to use
 
-        :type folder_id: str
-        :param folder_id: filter for folder by folder id
-
-          .. deprecated:: 0.16.0
-             To get details of a folder for which you know the ID, use the much
-             more efficient :meth:`show_folder` instead.
-
         :type name: str
         :param name: Folder name to filter on. For ``name`` specify the full
                      path of the folder starting from the library's root
@@ -271,36 +286,29 @@ class LibraryClient(Client):
 
         :rtype: list
         :return: list of dicts each containing basic information about a folder
+
+        .. versionchanged:: 1.1.0
+           Using the deprecated ``folder_id`` parameter now raises a
+           ``ValueError`` exception.
         """
         if folder_id is not None:
-            warnings.warn(
-                "The folder_id parameter is deprecated, use the show_folder() method to view details of a folder for which you know the ID.",
-                category=FutureWarning,
+            raise ValueError(
+                "The folder_id parameter has been removed, use the show_folder() method to view details of a folder for which you know the ID."
             )
-        if folder_id is not None and name is not None:
-            raise ValueError("Provide only one argument between name or folder_id, but not both")
         library_contents = self.show_library(library_id=library_id, contents=True)
-        if folder_id is not None:
-            folder = next((_ for _ in library_contents if _["type"] == "folder" and _["id"] == folder_id), None)
-            folders = [folder] if folder is not None else []
-        elif name is not None:
+        if name is not None:
             folders = [_ for _ in library_contents if _["type"] == "folder" and _["name"] == name]
         else:
             folders = [_ for _ in library_contents if _["type"] == "folder"]
         return folders
 
-    def get_libraries(self, library_id=None, name=None, deleted=False):
+    def get_libraries(
+        self, library_id: Optional[str] = None, name: Optional[str] = None, deleted: Optional[bool] = False
+    ) -> List[Dict[str, Any]]:
         """
         Get all libraries, or select a subset by specifying optional arguments
         for filtering (e.g. a library name).
 
-        :type library_id: str
-        :param library_id: filter for library by library id
-
-          .. deprecated:: 0.16.0
-             To get details of a library for which you know the ID, use the much
-             more efficient :meth:`show_library` instead.
-
         :type name: str
         :param name: Library name to filter on.
 
@@ -311,23 +319,21 @@ class LibraryClient(Client):
 
         :rtype: list
         :return: list of dicts each containing basic information about a library
+
+        .. versionchanged:: 1.1.0
+           Using the deprecated ``library_id`` parameter now raises a
+           ``ValueError`` exception.
         """
         if library_id is not None:
-            warnings.warn(
-                "The library_id parameter is deprecated, use the show_library() method to view details of a library for which you know the ID.",
-                category=FutureWarning,
+            raise ValueError(
+                "The library_id parameter has been removed, use the show_library() method to view details of a library for which you know the ID."
             )
-        if library_id is not None and name is not None:
-            raise ValueError("Provide only one argument between name or library_id, but not both")
         libraries = self._get(params={"deleted": deleted})
-        if library_id is not None:
-            library = next((_ for _ in libraries if _["id"] == library_id), None)
-            libraries = [library] if library is not None else []
         if name is not None:
             libraries = [_ for _ in libraries if _["name"] == name]
         return libraries
 
-    def show_library(self, library_id, contents=False):
+    def show_library(self, library_id: str, contents: bool = False) -> Dict[str, Any]:
         """
         Get information about a library.
 
@@ -343,55 +349,64 @@ class LibraryClient(Client):
         """
         return self._get(id=library_id, contents=contents)
 
-    def _do_upload(self, library_id, **keywords):
+    def _do_upload(self, library_id: str, **kwargs: Any) -> List[Dict[str, Any]]:
         """
         Set up the POST request and do the actual data upload to a data library.
         This method should not be called directly but instead refer to the
         methods specific for the desired type of data upload.
         """
-        folder_id = keywords.get("folder_id", None)
+        folder_id = kwargs.get("folder_id")
         if folder_id is None:
             folder_id = self._get_root_folder_id(library_id)
         files_attached = False
         # Compose the payload dict
-        payload = {}
-        payload["folder_id"] = folder_id
-        payload["file_type"] = keywords.get("file_type", "auto")
-        payload["dbkey"] = keywords.get("dbkey", "?")
-        payload["create_type"] = "file"
-        if keywords.get("roles", None):
-            payload["roles"] = keywords["roles"]
-        if keywords.get("link_data_only", None) and keywords["link_data_only"] != "copy_files":
+        payload = {
+            "folder_id": folder_id,
+            "file_type": kwargs.get("file_type", "auto"),
+            "dbkey": kwargs.get("dbkey", "?"),
+            "create_type": "file",
+            "tag_using_filenames": kwargs.get("tag_using_filenames", False),
+            "preserve_dirs": kwargs.get("preserve_dirs", False),
+        }
+        if kwargs.get("roles"):
+            payload["roles"] = kwargs["roles"]
+        if kwargs.get("link_data_only") and kwargs["link_data_only"] != "copy_files":
             payload["link_data_only"] = "link_to_files"
-        payload["tag_using_filenames"] = keywords.get("tag_using_filenames", False)
-        if keywords.get("tags"):
-            payload["tags"] = keywords["tags"]
-        payload["preserve_dirs"] = keywords.get("preserve_dirs", False)
+        if kwargs.get("tags"):
+            payload["tags"] = kwargs["tags"]
         # upload options
-        if keywords.get("file_url", None) is not None:
+        if kwargs.get("file_url") is not None:
             payload["upload_option"] = "upload_file"
-            payload["files_0|url_paste"] = keywords["file_url"]
-        elif keywords.get("pasted_content", None) is not None:
+            payload["files_0|url_paste"] = kwargs["file_url"]
+        elif kwargs.get("pasted_content") is not None:
             payload["upload_option"] = "upload_file"
-            payload["files_0|url_paste"] = keywords["pasted_content"]
-        elif keywords.get("server_dir", None) is not None:
+            payload["files_0|url_paste"] = kwargs["pasted_content"]
+        elif kwargs.get("server_dir") is not None:
             payload["upload_option"] = "upload_directory"
-            payload["server_dir"] = keywords["server_dir"]
-        elif keywords.get("file_local_path", None) is not None:
+            payload["server_dir"] = kwargs["server_dir"]
+        elif kwargs.get("file_local_path") is not None:
             payload["upload_option"] = "upload_file"
-            payload["files_0|file_data"] = attach_file(keywords["file_local_path"])
+            payload["files_0|file_data"] = attach_file(kwargs["file_local_path"])
             files_attached = True
-        elif keywords.get("filesystem_paths", None) is not None:
+        elif kwargs.get("filesystem_paths") is not None:
             payload["upload_option"] = "upload_paths"
-            payload["filesystem_paths"] = keywords["filesystem_paths"]
+            payload["filesystem_paths"] = kwargs["filesystem_paths"]
 
         try:
             return self._post(payload, id=library_id, contents=True, files_attached=files_attached)
         finally:
-            if payload.get("files_0|file_data", None) is not None:
+            if payload.get("files_0|file_data") is not None:
                 payload["files_0|file_data"].close()
 
-    def upload_file_from_url(self, library_id, file_url, folder_id=None, file_type="auto", dbkey="?", tags=None):
+    def upload_file_from_url(
+        self,
+        library_id: str,
+        file_url: str,
+        folder_id: Optional[str] = None,
+        file_type: str = "auto",
+        dbkey: str = "?",
+        tags: Optional[List[str]] = None,
+    ) -> List[Dict[str, Any]]:
         """
         Upload a file to a library from a URL.
 
@@ -421,7 +436,15 @@ class LibraryClient(Client):
             library_id, file_url=file_url, folder_id=folder_id, file_type=file_type, dbkey=dbkey, tags=tags
         )
 
-    def upload_file_contents(self, library_id, pasted_content, folder_id=None, file_type="auto", dbkey="?", tags=None):
+    def upload_file_contents(
+        self,
+        library_id: str,
+        pasted_content: str,
+        folder_id: Optional[str] = None,
+        file_type: str = "auto",
+        dbkey: str = "?",
+        tags: Optional[List[str]] = None,
+    ) -> List[Dict[str, Any]]:
         """
         Upload pasted_content to a data library as a new file.
 
@@ -452,8 +475,14 @@ class LibraryClient(Client):
         )
 
     def upload_file_from_local_path(
-        self, library_id, file_local_path, folder_id=None, file_type="auto", dbkey="?", tags=None
-    ):
+        self,
+        library_id: str,
+        file_local_path: str,
+        folder_id: Optional[str] = None,
+        file_type: str = "auto",
+        dbkey: str = "?",
+        tags: Optional[List[str]] = None,
+    ) -> List[Dict[str, Any]]:
         """
         Read local file contents from file_local_path and upload data to a
         library.
@@ -491,17 +520,17 @@ class LibraryClient(Client):
 
     def upload_file_from_server(
         self,
-        library_id,
-        server_dir,
-        folder_id=None,
-        file_type="auto",
-        dbkey="?",
-        link_data_only=None,
-        roles="",
-        preserve_dirs=False,
-        tag_using_filenames=False,
-        tags=None,
-    ):
+        library_id: str,
+        server_dir: str,
+        folder_id: Optional[str] = None,
+        file_type: str = "auto",
+        dbkey: str = "?",
+        link_data_only: Optional[LinkDataOnly] = None,
+        roles: str = "",
+        preserve_dirs: bool = False,
+        tag_using_filenames: bool = False,
+        tags: Optional[List[str]] = None,
+    ) -> List[Dict[str, Any]]:
         """
         Upload all files in the specified subdirectory of the Galaxy library
         import directory to a library.
@@ -569,17 +598,17 @@ class LibraryClient(Client):
 
     def upload_from_galaxy_filesystem(
         self,
-        library_id,
-        filesystem_paths,
-        folder_id=None,
-        file_type="auto",
-        dbkey="?",
-        link_data_only=None,
-        roles="",
-        preserve_dirs=False,
-        tag_using_filenames=False,
-        tags=None,
-    ):
+        library_id: str,
+        filesystem_paths: str,
+        folder_id: Optional[str] = None,
+        file_type: str = "auto",
+        dbkey: str = "?",
+        link_data_only: Optional[LinkDataOnly] = None,
+        roles: str = "",
+        preserve_dirs: bool = False,
+        tag_using_filenames: bool = False,
+        tags: Optional[List[str]] = None,
+    ) -> List[Dict[str, Any]]:
         """
         Upload a set of files already present on the filesystem of the Galaxy
         server to a library.
@@ -623,7 +652,8 @@ class LibraryClient(Client):
         :param tags: A list of tags to add to the datasets
 
         :rtype: list
-        :return: List with a single dictionary containing information about the LDDA
+        :return: List of dictionaries containing information about each uploaded
+          LDDA.
 
         .. note::
           This method works only if the Galaxy instance has the
@@ -643,7 +673,9 @@ class LibraryClient(Client):
             tags=tags,
         )
 
-    def copy_from_dataset(self, library_id, dataset_id, folder_id=None, message=""):
+    def copy_from_dataset(
+        self, library_id: str, dataset_id: str, folder_id: Optional[str] = None, message: str = ""
+    ) -> Dict[str, Any]:
         """
         Copy a Galaxy dataset into a library.
 
@@ -665,14 +697,15 @@ class LibraryClient(Client):
         """
         if folder_id is None:
             folder_id = self._get_root_folder_id(library_id)
-        payload = {}
-        payload["folder_id"] = folder_id
-        payload["create_type"] = "file"
-        payload["from_hda_id"] = dataset_id
-        payload["ldda_message"] = message
+        payload = {
+            "folder_id": folder_id,
+            "create_type": "file",
+            "from_hda_id": dataset_id,
+            "ldda_message": message,
+        }
         return self._post(payload, id=library_id, contents=True)
 
-    def get_library_permissions(self, library_id):
+    def get_library_permissions(self, library_id: str) -> Dict[str, Any]:
         """
         Get the permissions for a library.
 
@@ -685,7 +718,7 @@ class LibraryClient(Client):
         url = self._make_url(library_id) + "/permissions"
         return self._get(url=url)
 
-    def get_dataset_permissions(self, dataset_id):
+    def get_dataset_permissions(self, dataset_id: str) -> Dict[str, Any]:
         """
         Get the permissions for a dataset.
 
@@ -698,7 +731,14 @@ class LibraryClient(Client):
         url = "/".join((self._make_url(), "datasets", dataset_id, "permissions"))
         return self._get(url=url)
 
-    def set_library_permissions(self, library_id, access_in=None, modify_in=None, add_in=None, manage_in=None):
+    def set_library_permissions(
+        self,
+        library_id: str,
+        access_in: Optional[List[str]] = None,
+        modify_in: Optional[List[str]] = None,
+        add_in: Optional[List[str]] = None,
+        manage_in: Optional[List[str]] = None,
+    ) -> Dict[str, Any]:
         """
         Set the permissions for a library. Note: it will override all security
         for this library even if you leave out a permission type.
@@ -721,7 +761,7 @@ class LibraryClient(Client):
         :rtype: dict
         :return: General information about the library
         """
-        payload = {}
+        payload: Dict[str, List[str]] = {}
         if access_in:
             payload["LIBRARY_ACCESS_in"] = access_in
         if modify_in:
@@ -733,7 +773,13 @@ class LibraryClient(Client):
         url = self._make_url(library_id) + "/permissions"
         return self._post(payload, url=url)
 
-    def set_dataset_permissions(self, dataset_id, access_in=None, modify_in=None, manage_in=None):
+    def set_dataset_permissions(
+        self,
+        dataset_id: str,
+        access_in: Optional[List[str]] = None,
+        modify_in: Optional[List[str]] = None,
+        manage_in: Optional[List[str]] = None,
+    ) -> Dict[str, Any]:
         """
         Set the permissions for a dataset. Note: it will override all security
         for this dataset even if you leave out a permission type.
@@ -753,14 +799,15 @@ class LibraryClient(Client):
         :rtype: dict
         :return: dictionary with all applicable permissions' values
         """
-        payload = {}
+        # we need here to define an action
+        payload: Dict[str, Any] = {
+            "action": "set_permissions",
+        }
         if access_in:
             payload["access_ids[]"] = access_in
         if modify_in:
             payload["modify_ids[]"] = modify_in
         if manage_in:
             payload["manage_ids[]"] = manage_in
-        # we need here to define an action
-        payload["action"] = "set_permissions"
         url = "/".join((self._make_url(), "datasets", dataset_id, "permissions"))
         return self._post(payload, url=url)
