@@ -1,36 +1,41 @@
 """
 Contains possible interaction dealing with Galaxy tools.
 """
-import warnings
 from os.path import basename
 from typing import (
+    Any,
+    Dict,
     List,
     Optional,
+    TYPE_CHECKING,
+    Union,
 )
 
+from typing_extensions import Literal
+
 from bioblend.galaxy.client import Client
 from bioblend.galaxyclient import UPLOAD_CHUNK_SIZE
 from bioblend.util import attach_file
+from .inputs import InputsBuilder
+
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
 
 
 class ToolClient(Client):
+    gi: "GalaxyInstance"
     module = "tools"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def get_tools(self, tool_id=None, name=None, trackster=None):
+    def get_tools(
+        self, tool_id: Optional[str] = None, name: Optional[str] = None, trackster: Optional[bool] = None
+    ) -> List[Dict[str, Any]]:
         """
         Get all tools, or select a subset by specifying optional arguments for
         filtering (e.g. a tool name).
 
-        :type tool_id: str
-        :param tool_id: id of the requested tool
-
-          .. deprecated:: 0.16.0
-             To get details of a tool for which you know the ID, use the much
-             more efficient :meth:`show_tool` instead.
-
         :type name: str
         :param name: Tool name to filter on.
 
@@ -42,23 +47,21 @@ class ToolClient(Client):
         :return: List of tool descriptions.
 
         .. seealso:: bioblend.galaxy.toolshed.get_repositories()
+
+        .. versionchanged:: 1.1.0
+           Using the deprecated ``tool_id`` parameter now raises a
+           ``ValueError`` exception.
         """
         if tool_id is not None:
-            warnings.warn(
-                "The tool_id parameter is deprecated, use the show_tool() method to view details of a tool for which you know the ID.",
-                category=FutureWarning,
+            raise ValueError(
+                "The tool_id parameter has been removed, use the show_tool() method to view details of a tool for which you know the ID."
             )
-        if tool_id is not None and name is not None:
-            raise ValueError("Provide only one argument between name or tool_id, but not both")
         tools = self._raw_get_tool(in_panel=False, trackster=trackster)
-        if tool_id is not None:
-            tool = next((_ for _ in tools if _["id"] == tool_id), None)
-            tools = [tool] if tool is not None else []
-        elif name is not None:
+        if name is not None:
             tools = [_ for _ in tools if _["name"] == name]
         return tools
 
-    def get_tool_panel(self):
+    def get_tool_panel(self) -> List[Dict[str, Any]]:
         """
         Get a list of available tool elements in Galaxy's configured toolbox.
 
@@ -70,13 +73,14 @@ class ToolClient(Client):
         """
         return self._raw_get_tool(in_panel=True)
 
-    def _raw_get_tool(self, in_panel=None, trackster=None):
-        params = {}
-        params["in_panel"] = in_panel
-        params["trackster"] = trackster
+    def _raw_get_tool(self, in_panel: Optional[bool] = None, trackster: Optional[bool] = None) -> List[Dict[str, Any]]:
+        params = {
+            "in_panel": in_panel,
+            "trackster": trackster,
+        }
         return self._get(params=params)
 
-    def requirements(self, tool_id):
+    def requirements(self, tool_id: str) -> List[Dict[str, Any]]:
         """
         Return the resolver status for a specific tool.
 
@@ -148,7 +152,7 @@ class ToolClient(Client):
         url = self._make_url(tool_id) + "/citations"
         return self._get(url=url)
 
-    def install_dependencies(self, tool_id):
+    def install_dependencies(self, tool_id: str) -> List[Dict[str, Any]]:
         """
         Install dependencies for a given tool via a resolver.
         This works only for Conda currently.
@@ -156,8 +160,8 @@ class ToolClient(Client):
         :type tool_id: str
         :param tool_id: id of the requested tool
 
-        :rtype: dict
-        :return: Tool requirement status
+        :rtype: list of dicts
+        :return: List of tool requirement status dictionaries
 
         .. note::
           This method works only if the user is a Galaxy admin.
@@ -182,7 +186,7 @@ class ToolClient(Client):
         url = self._make_url(tool_id) + "/dependencies"
         return self._delete(url=url)
 
-    def show_tool(self, tool_id, io_details=False, link_details=False):
+    def show_tool(self, tool_id: str, io_details: bool = False, link_details: bool = False) -> Dict[str, Any]:
         """
         Get details of a given tool.
 
@@ -198,12 +202,19 @@ class ToolClient(Client):
         :rtype: dict
         :return: Information about the tool's interface
         """
-        params = {}
-        params["io_details"] = io_details
-        params["link_details"] = link_details
+        params = {
+            "io_details": io_details,
+            "link_details": link_details,
+        }
         return self._get(id=tool_id, params=params)
 
-    def build(self, tool_id, inputs=None, tool_version=None, history_id=None):
+    def build(
+        self,
+        tool_id: str,
+        inputs: Optional[Dict[str, Any]] = None,
+        tool_version: Optional[str] = None,
+        history_id: Optional[str] = None,
+    ) -> Dict[str, Any]:
         """
         This method returns the tool model, which includes an updated input parameter array for the given tool,
         based on user-defined "inputs".
@@ -301,7 +312,7 @@ class ToolClient(Client):
             }
 
         """
-        params = {}
+        params: Dict[str, Union[str, Dict]] = {}
 
         if inputs:
             params["inputs"] = inputs
@@ -316,7 +327,13 @@ class ToolClient(Client):
 
         return self._post(payload=params, url=url)
 
-    def run_tool(self, history_id, tool_id, tool_inputs, input_format="legacy"):
+    def run_tool(
+        self,
+        history_id: str,
+        tool_id: str,
+        tool_inputs: Union[InputsBuilder, dict],
+        input_format: Literal["21.01", "legacy"] = "legacy",
+    ) -> Dict[str, Any]:
         """
         Runs tool specified by ``tool_id`` in history indicated
         by ``history_id`` with inputs from ``dict`` ``tool_inputs``.
@@ -389,13 +406,15 @@ class ToolClient(Client):
         You can also check the examples in `Galaxy's API test suite
         <https://github.com/galaxyproject/galaxy/blob/dev/lib/galaxy_test/api/test_tools.py>`_.
         """
-        payload = {}
-        payload["history_id"] = history_id
-        payload["tool_id"] = tool_id
-        payload["input_format"] = input_format
-        try:
+        payload: Dict[str, Union[str, Dict]] = {
+            "history_id": history_id,
+            "tool_id": tool_id,
+            "input_format": input_format,
+        }
+
+        if isinstance(tool_inputs, InputsBuilder):
             payload["inputs"] = tool_inputs.to_dict()
-        except AttributeError:
+        else:
             payload["inputs"] = tool_inputs
         return self._post(payload)
 
@@ -406,8 +425,8 @@ class ToolClient(Client):
         storage: Optional[str] = None,
         metadata: Optional[dict] = None,
         chunk_size: Optional[int] = UPLOAD_CHUNK_SIZE,
-        **keywords,
-    ):
+        **kwargs: Any,
+    ) -> Dict[str, Any]:
         """
         Upload the file specified by ``path`` to the history specified by
         ``history_id``.
@@ -454,25 +473,25 @@ class ToolClient(Client):
         :return: Information about the created upload job
 
         .. note::
-          The following options work only on Galaxy 22.01 or later: ``storage``,
-          ``metadata``, ``chunk_size``, ``auto_decompress``.
+          The following parameters work only on Galaxy 22.01 or later:
+          ``storage``, ``metadata``, ``chunk_size``, ``auto_decompress``.
         """
         if self.gi.config.get_version()["version_major"] >= "22.01":
             # Use the tus protocol
             uploader = self.gi.get_tus_uploader(path, storage=storage, metadata=metadata, chunk_size=chunk_size)
             uploader.upload()
-            return self.post_to_fetch(path, history_id, uploader.session_id, **keywords)
+            return self.post_to_fetch(path, history_id, uploader.session_id, **kwargs)
         else:
-            if "file_name" not in keywords:
-                keywords["file_name"] = basename(path)
-            payload = self._upload_payload(history_id, **keywords)
-            payload["files_0|file_data"] = attach_file(path, name=keywords["file_name"])
+            if "file_name" not in kwargs:
+                kwargs["file_name"] = basename(path)
+            payload = self._upload_payload(history_id, **kwargs)
+            payload["files_0|file_data"] = attach_file(path, name=kwargs["file_name"])
             try:
                 return self._post(payload, files_attached=True)
             finally:
                 payload["files_0|file_data"].close()
 
-    def post_to_fetch(self, path: str, history_id: str, session_id: str, **keywords):
+    def post_to_fetch(self, path: str, history_id: str, session_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Make a POST request to the Fetch API after performing a tus upload.
 
@@ -494,11 +513,11 @@ class ToolClient(Client):
         :rtype: dict
         :return: Information about the created upload job
         """
-        payload = self._fetch_payload(path, history_id, session_id, **keywords)
+        payload = self._fetch_payload(path, history_id, session_id, **kwargs)
         url = "/".join((self.gi.url, "tools/fetch"))
         return self._post(payload, url=url)
 
-    def upload_from_ftp(self, path, history_id, **keywords):
+    def upload_from_ftp(self, path: str, history_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Upload the file specified by ``path`` from the user's FTP directory to
         the history specified by ``history_id``.
@@ -514,11 +533,11 @@ class ToolClient(Client):
         :rtype: dict
         :return: Information about the created upload job
         """
-        payload = self._upload_payload(history_id, **keywords)
+        payload = self._upload_payload(history_id, **kwargs)
         payload["files_0|ftp_files"] = path
         return self._post(payload)
 
-    def paste_content(self, content, history_id, **kwds):
+    def paste_content(self, content: str, history_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Upload a string to a new dataset in the history specified by
         ``history_id``.
@@ -535,37 +554,38 @@ class ToolClient(Client):
 
         See :meth:`upload_file` for the optional parameters.
         """
-        payload = self._upload_payload(history_id, **kwds)
+        payload = self._upload_payload(history_id, **kwargs)
         payload["files_0|url_paste"] = content
         return self._post(payload, files_attached=False)
 
     put_url = paste_content
 
-    def _upload_payload(self, history_id, **keywords):
-        payload = {}
-        payload["history_id"] = history_id
-        payload["tool_id"] = keywords.get("tool_id", "upload1")
-        tool_input = {}
-        tool_input["file_type"] = keywords.get("file_type", "auto")
-        tool_input["dbkey"] = keywords.get("dbkey", "?")
-        if not keywords.get("to_posix_lines", True):
+    def _upload_payload(self, history_id: str, **kwargs: Any) -> Dict[str, Any]:
+        tool_input: Dict[str, Any] = {
+            "file_type": kwargs.get("file_type", "auto"),
+            "dbkey": kwargs.get("dbkey", "?"),
+            "files_0|type": "upload_dataset",
+        }
+        if not kwargs.get("to_posix_lines", True):
             tool_input["files_0|to_posix_lines"] = False
-        elif keywords.get("space_to_tab", False):
+        elif kwargs.get("space_to_tab", False):
             tool_input["files_0|space_to_tab"] = "Yes"
-        if "file_name" in keywords:
-            tool_input["files_0|NAME"] = keywords["file_name"]
-        tool_input["files_0|type"] = "upload_dataset"
-        payload["inputs"] = tool_input
-        return payload
+        if "file_name" in kwargs:
+            tool_input["files_0|NAME"] = kwargs["file_name"]
+        return {
+            "history_id": history_id,
+            "tool_id": kwargs.get("tool_id", "upload1"),
+            "inputs": tool_input,
+        }
 
-    def _fetch_payload(self, path: str, history_id: str, session_id: str, **keywords) -> dict:
-        file_name = keywords.get("file_name", basename(path))
+    def _fetch_payload(self, path: str, history_id: str, session_id: str, **kwargs: Any) -> dict:
+        file_name = kwargs.get("file_name", basename(path))
         element = {
             "src": "files",
-            "ext": keywords.get("file_type", "auto"),
-            "dbkey": keywords.get("dbkey", "?"),
-            "to_posix_lines": keywords.get("to_posix_lines", True),
-            "space_to_tab": keywords.get("space_to_tab", False),
+            "ext": kwargs.get("file_type", "auto"),
+            "dbkey": kwargs.get("dbkey", "?"),
+            "to_posix_lines": kwargs.get("to_posix_lines", True),
+            "space_to_tab": kwargs.get("space_to_tab", False),
             "name": file_name,
         }
         payload = {
@@ -577,6 +597,6 @@ class ToolClient(Client):
                 }
             ],
             "files_0|file_data": {"session_id": session_id, "name": file_name},
-            "auto_decompress": keywords.get("auto_decompress", False),
+            "auto_decompress": kwargs.get("auto_decompress", False),
         }
         return payload
